use std::sync::mpsc;
use std::thread;
use std::{collections::HashMap, process::Command};

use anyhow::{Context, Result};
use futures::{SinkExt, StreamExt};
use tokio::runtime::Runtime;
use tokio_tungstenite::connect_async;
use tungstenite::Message as WsMessage;
use url::Url;

use crate::asciicast::HeaderBuilder;
use crate::client::terminal::Terminal;
use crate::message::Message;

#[derive(Debug, Parser)]
pub struct StreamOpts {
  /// The broadcast websocket address of the server
  #[clap(long = "broadcast-url", env = "LIVETERM_BROADCAST_URL")]
  broadcast_url: Url,
}

pub fn stream_main(opts: StreamOpts) -> Result<()> {
  let runtime = Runtime::new()?;

  runtime.block_on(stream_async_main(opts))
}

async fn stream_async_main(opts: StreamOpts) -> Result<()> {
  println!("Hellosu {opts:?}");

  // Open a new websocket connection to the server
  // let url = Url::parse(&"ws://localhost:8200/broadcast").unwrap();

  let (ws, _) = connect_async(&opts.broadcast_url)
    .await
    .context("Failed to connect")?;
  println!("WebSocket handshake has been successfully completed");

  let (mut write, mut read) = ws.split();

  let mut command = Command::new("zsh");
  command.env("TERM", "xterm-256color");

  // Write header
  // TODO: Clean this up
  let header = {
    let command_str = format!("{command:?}");
    let env = command
      .get_envs()
      .into_iter()
      .filter_map(|(a, b)| b.map(|b| (a, b)))
      .map(|(a, b)| {
        (
          a.to_string_lossy().to_string(),
          b.to_string_lossy().to_string(),
        )
      })
      .collect::<HashMap<_, _>>();

    HeaderBuilder::default()
      .width(30)
      .height(30)
      .command(command_str)
      .env(env)
      .build()?
  };

  // Step 1. Send header
  {
    let message = Message::AsciicastHeader(header);
    let ws_message = WsMessage::Text(serde_json::to_string(&message).unwrap());
    write.send(ws_message).await?;
  }

  // STep 2. Wait for hellosu
  {
    let msg = match read.next().await {
      Some(v) => v?,
      None => bail!("wtf bro"),
    };
    let msg: Message = match msg {
      WsMessage::Text(v) => serde_json::from_str(&v)?,
      _ => bail!("dont send me other Shit!!!"),
    };
    let server_hello = match msg {
      Message::ServerHello(v) => v,
      _ => bail!("DONT SEND ME OTHER SHIT"),
    };
    println!("URL: {:?}", server_hello.url);
  }

  // Step 3. Produuuuuuce
  let (tx, rx) = mpsc::channel();
  let (pty, rxt) = Terminal::setup(command)?;
  thread::spawn(move || {
    for event in rxt.into_iter() {
      let message = Message::AsciicastEvent(event);
      let ws_message =
        WsMessage::Text(serde_json::to_string(&message).unwrap());
      tx.send(ws_message);
    }
  });

  tokio::spawn(async move {
    for ws_message in rx.into_iter() {
      write.send(ws_message).await;
    }
  });

  pty.wait_until_complete()?;

  Ok(())
}
