use anyhow::Result;
use axum::{
  extract::{
    ws::{Message as WsMessage, WebSocket},
    Path, WebSocketUpgrade,
  },
  http::StatusCode,
  response::Response,
};
use futures::{future::BoxFuture, FutureExt};
use tokio::sync::broadcast::Receiver;

use crate::message::Message;

use super::broadcast::BROADCASTS;

pub async fn listen(
  Path((room_id,)): Path<(String,)>,
  ws: WebSocketUpgrade,
) -> Result<Response, StatusCode> {
  let rx = match BROADCASTS.get(&room_id) {
    Some(v) => (*v).subscribe(),
    None => return Err(StatusCode::NOT_FOUND),
  };

  Ok(ws.on_upgrade(handle_websocket(rx)))
}

async fn handle_ws_page_inner(
  mut rx: Receiver<Message>,
  mut socket: WebSocket,
) -> Result<()> {
  info!("Started WS connection from website.");

  while let Ok(message) = rx.recv().await {
    let ws_message = WsMessage::Text(serde_json::to_string(&message)?);
    info!(
      "[DM->Web] Received WS message from dashmap, forwarding {ws_message:?}"
    );
    socket.send(ws_message).await?;
  }

  Ok(())
}

fn handle_websocket<'a>(
  rx: Receiver<Message>,
) -> impl FnOnce(WebSocket) -> BoxFuture<'a, ()> {
  move |socket: WebSocket| {
    async {
      match handle_ws_page_inner(rx, socket).await {
        Ok(_) => {}
        Err(err) => {
          eprintln!("Fuck! {err:?}");
          eprintln!("{err}");
        }
      }
    }
    .boxed()
  }
}
